#pragma once

#include "gamefield.h"
#include <vector>

namespace AI {
    struct Moving
    {
        enum {
            MOV_NULL,
            MOV_L,
            MOV_R,
            MOV_LL,
            MOV_RR,
            MOV_D,
            MOV_DD,
            MOV_LSPIN,
            MOV_RSPIN,
            MOV_DROP,
            MOV_HOLD,
            MOV_SPIN2,
        };
        std::vector<int> movs;
        int x, y;
        int score, score2;
        signed char spin;
        signed char wallkick_spin;
        Moving () { wallkick_spin = 0; movs.reserve (16); }
        Moving ( const Moving & m ) {
            movs = m.movs;
            x = m.x;
            y = m.y;
            spin = m.spin;
            score = m.score;
            score2 = m.score2;
            wallkick_spin = m.wallkick_spin;
        }
        Moving ( const Moving & m, int _spin ) {
            movs.reserve (16);
            movs = m.movs;
            spin = (signed char)_spin;
        }
        bool operator < (const Moving& _m) const {
            return score > _m.score;
        }
        bool operator == (const Moving& _m) const {
            if ( x != _m.x || y != _m.y ) return false;
            if ( spin != _m.spin ) return false;
            if ( wallkick_spin != _m.wallkick_spin ) return false;
            return true;
        }
    };
    template <class T>
    struct MovList
    {
        std::vector<T> queue;
        //T queue[1024];
        int beg, end;
        MovList() {
            beg = end = 0;
        }
        MovList( size_t size ) {
            beg = end = 0;
            //queue.resize( size );
            queue.reserve( size );
        }
        void clear() {
            beg = end = 0;
            queue.clear();
        }
        size_t size() const {
            return end - beg;
        }
        T& back() {
            return queue[end-1];
        }
        void push(const T& mov) {
            queue.push_back(mov);
            //queue[end] = mov;
            ++end;
        }
        bool empty () const {
            return beg == end;
        }
        void pop(T& mov) {
            mov = queue[beg++];
        }
    };
    void setComboList( std::vector<int> combolist );
    int getComboAttack( int combo );
    Moving AISearch(const GameField& field, Gem cur, int x, int y, const std::vector<Gem>& next, bool canhold, int upcomeAtt, int level, int player);
    void setSpin180( bool enable );
}
